==> og1_ui-0.0.15.tar.gz: lib/src/widgets/thememode_selector/og1_thememode_selector.dart <==
import 'package:flutter/material.dart';
import 'package:og1_ui/src/widgets/thememode_selector/celestial_transition.dart';
import 'package:og1_ui/src/widgets/thememode_selector/flare.dart';
import 'package:og1_ui/src/widgets/thememode_selector/focus_background.dart';
import 'package:og1_ui/src/widgets/thememode_selector/moon.dart';
import 'package:og1_ui/src/widgets/thememode_selector/star.dart';
import 'package:og1_ui/src/widgets/thememode_selector/sun.dart';
import 'package:og1_ui/src/widgets/thememode_selector/thememode_selector_consts.dart';
import 'package:og1_ui/src/widgets/thememode_selector/thememode_selector_theme.dart';

class Og1ThemeModeSelector extends StatefulWidget {
  final int durationInMs;
  final Color? lightBackgroundColor;
  final Color? darkBackgroundColor;
  final Color? lightToggleColor;
  final Color? darkToggleColor;
  final double height;
  final ValueChanged<ThemeMode> onChanged;
  final ThemeMode themeMode;

  /// Creates a ThemeMode Selector.
  ///
  /// This selector maintains its own state and the widget calls the
  /// [onChanged] callback when its state is changed.
  ///
  /// * [height] allows the user to control the height of the widget and
  ///    a default height of 39 is used.
  /// * [onChanged] is called while the user is selecting a new value for the
  ///   slider.
  /// * [lightBackground] and [lightToggle] are colors which control the
  ///   foreground and background colors representing the "light" theme mode
  /// * [darkBackground] and [darkToggle] are colors which control the
  ///   foreground and background colors representing the "dark" theme mode
  ///
  const Og1ThemeModeSelector({
    Key? key,
    this.durationInMs = 750,
    this.lightBackgroundColor,
    this.lightToggleColor,
    this.darkBackgroundColor,
    this.darkToggleColor,
    this.height = 39,
    required this.onChanged,
    required this.themeMode,
  }) : super(key: key);

  @override
  _Og1ThemeModeSelectorState createState() => _Og1ThemeModeSelectorState();
}

class _Og1ThemeModeSelectorState extends State<Og1ThemeModeSelector>
    with TickerProviderStateMixin {
  late ThemeModeSelectorConsts consts;
  late AnimationController _animationController;
  final Set<MaterialState> _states = {};

  late Animation<Alignment> _alignmentAnimation;
  late Animation<double> _starFade;
  late Animation<double> _flareFade;
  late Animation<double> _starToggleFade;
  late Animation<double> _flareToggleFade;
  late Animation<Color?> _bgColorAnimation;

  late bool isChecked;

  @override
  void initState() {
    super.initState();
    consts = ThemeModeSelectorConsts(widget.height);

    // Setup the global animation controller using the duration parameter
    _initializeAnimationController();
    isChecked = widget.themeMode == ThemeMode.dark;
    if (isChecked) {
      _animationController.forward();
    }
  }

  @override
  void didUpdateWidget(Og1ThemeModeSelector oldWidget) {
    super.didUpdateWidget(oldWidget);

    bool isCheckedUpdated = widget.themeMode == ThemeMode.dark;
    if (widget.themeMode != oldWidget.themeMode &&
        isChecked != isCheckedUpdated) {
      // rebuild the global animation controller using the duration parameter
      _initializeAnimationController();
      setState(() => isChecked = isCheckedUpdated);
      if (isCheckedUpdated) {
        _animationController.forward();
      } else {
        _animationController.reverse();
      }
    }
  }

  _initializeAnimationController() {
    Duration _duration = Duration(milliseconds: widget.durationInMs);

    _animationController = AnimationController(
      vsync: this,
      duration: _duration,
    );
  }

  initialize(BuildContext context, ThemeModeSelectorThemeData myTheme) {
    // Setup the tween for the background colors
    _bgColorAnimation = ColorTween(
      begin: lightBackgroundColor(myTheme),
      end: darkBackgroundColor(myTheme),
    ).animate(_animationController);

    // the tween for the toggle button (left and right)
    _alignmentAnimation = AlignmentTween(
      begin: Alignment.centerLeft,
      end: Alignment.centerRight,
    ).animate(
      CurvedAnimation(
        parent: _animationController,
        curve: const Interval(0.0, 0.9, curve: Curves.easeOutBack),
        reverseCurve: const Interval(0.0, 0.9, curve: Curves.easeInBack),
      ),
    );

    // Tweens and animation for the stars and flares
    var earlyFade = CurvedAnimation(
      parent: _animationController,
      curve: const Interval(0.0, 0.5, curve: Curves.elasticOut),
      reverseCurve: const Interval(0.5, 1.0, curve: Curves.elasticIn),
    );

    _starFade = Tween(begin: 0.0, end: 1.0).animate(earlyFade);
    _flareFade = Tween(begin: 1.0, end: 0.0).animate(earlyFade);
    _starToggleFade = Tween(begin: 0.0, end: 1.0).animate(earlyFade);
    _flareToggleFade = Tween(begin: 1.0, end: 0.0).animate(earlyFade);
  }

  @override
  void dispose() {
    _animationController.dispose();
    super.dispose();
  }

  // Builds the semi-complex tween for the stars and flares which aninate to
  // and fro from the center of the widget
  Animation<RelativeRect> slide(Offset from, Offset to, double size) {
    var container =
        Rect.fromLTWH(0, 0, consts.inset.width, consts.inset.height);
    return RelativeRectTween(
      begin: RelativeRect.fromRect(
          Rect.fromLTWH(from.dx, from.dy, size, size), container),
      end: RelativeRect.fromRect(
          Rect.fromLTWH(to.dx, to.dy, size, size), container),
    ).animate(
        // _animationController
        CurvedAnimation(
      parent: _animationController,
      curve: Curves.easeOutBack,
      reverseCurve: Curves.easeOut.flipped,
    ));
  }

  lightToggleColor(myTheme) =>
      widget.lightToggleColor ?? myTheme.lightToggleColor ?? Colors.white;

  lightBackgroundColor(myTheme) =>
      widget.lightBackgroundColor ??
      myTheme.lightBackgroundColor ??
      const Color(0xFF689DFF);

  darkToggleColor(myTheme) =>
      widget.darkToggleColor ?? myTheme.darkToggleColor ?? Colors.white;

  darkBackgroundColor(myTheme) =>
      widget.darkBackgroundColor ??
      myTheme.darkBackgroundColor ??
      const Color(0xFF040507);

  void _handleFocusHighlight(bool value) {
    bool modified = value
        ? _states.add(MaterialState.focused)
        : _states.remove(MaterialState.focused);
    if (modified) setState(() {});
  }

  void _handleHoverHighlight(bool value) {
    bool modified = value
        ? _states.add(MaterialState.hovered)
        : _states.remove(MaterialState.hovered);
    if (modified) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    ThemeModeSelectorThemeData myTheme = ThemeModeSelectorTheme.of(context);
    initialize(context, myTheme);

    return FocusableActionDetector(
      onShowFocusHighlight: _handleFocusHighlight,
      onShowHoverHighlight: _handleHoverHighlight,
      child: AnimatedBuilder(
        animation: _animationController,
        builder: (context, child) {
          return GestureDetector(
            onTap: () {
              if (_animationController.isCompleted) {
                _animationController.reverse();
                widget.onChanged(ThemeMode.light);
              } else {
                _animationController.forward();
                widget.onChanged(ThemeMode.dark);
              }
              setState(() {
                isChecked = !isChecked;
              });
            },
            child: Stack(
              alignment: Alignment.center,
              children: [
                Container(
                  width: consts.size.width,
                  height: consts.size.height,
                  padding: consts.padding,
                  decoration: BoxDecoration(
                    color: _bgColorAnimation.value,
                    borderRadius: BorderRadius.all(
                      Radius.circular(consts.size.height),
                    ),
                  ),
                  child: Stack(
                    children: <Widget>[
                      ...consts.stars
                          .map((star) => CelestialTransition(
                                alphaAnimation: _starFade,
                                child: Star(
                                    size: star['size'],
                                    color: lightToggleColor(myTheme)),
                                relativeRectAnimation: slide(
                                    star['from'], star['to'], star['size']),
                              ))
                          .toList(),
                      ...consts.flares
                          .map((flare) => CelestialTransition(
                                alphaAnimation: _flareFade,
                                child: Flare(
                                    size: flare['size'],
                                    color: lightToggleColor(myTheme)),
                                relativeRectAnimation: slide(
                                    flare['from'], flare['to'], flare['size']),
                              ))
                          .toList(),
                      Align(
                        alignment: _alignmentAnimation.value,
                        child: Stack(children: [
                          FadeTransition(
                            opacity: _flareToggleFade,
                            child: Sun(
                              color: lightToggleColor(myTheme),
                              size: consts.inset.height,
                            ),
                          ),
                          FadeTransition(
                            opacity: _starToggleFade,
                            child: Moon(
                              color: darkToggleColor(myTheme),
                              size: consts.inset.height,
                            ),
                          ),
                        ]),
                      ),
                    ],
                  ),
                ),
                FocusBackground(
                  padding: consts.focusPadding,
                  focused: _states.contains(MaterialState.focused),
                  width: consts.size.width,
                  height: consts.size.height,
                ),
              ],
            ),
          );
        },
      ),
    );
  }
}

==> og1_ui-0.0.23.tar.gz: lib/src/widgets/thememode_selector/og1_thememode_selector.dart <==
import 'package:flutter/material.dart';
import 'package:og1_ui/src/widgets/thememode_selector/celestial_transition.dart';
import 'package:og1_ui/src/widgets/thememode_selector/flare.dart';
import 'package:og1_ui/src/widgets/thememode_selector/focus_background.dart';
import 'package:og1_ui/src/widgets/thememode_selector/moon.dart';
import 'package:og1_ui/src/widgets/thememode_selector/star.dart';
import 'package:og1_ui/src/widgets/thememode_selector/sun.dart';
import 'package:og1_ui/src/widgets/thememode_selector/thememode_selector_consts.dart';
import 'package:og1_ui/src/widgets/thememode_selector/thememode_selector_theme.dart';

class Og1ThemeModeSelector extends StatefulWidget {
  final int durationInMs;
  final Color? lightBackgroundColor;
  final Color? darkBackgroundColor;
  final Color? lightToggleColor;
  final Color? darkToggleColor;
  final double height;
  final ValueChanged<ThemeMode> onChanged;
  final ThemeMode themeMode;

  /// Creates a ThemeMode Selector.
  ///
  /// This selector maintains its own state and the widget calls the
  /// [onChanged] callback when its state is changed.
  ///
  /// * [height] allows the user to control the height of the widget and
  ///    a default height of 39 is used.
  /// * [onChanged] is called while the user is selecting a new value for the
  ///   slider.
  /// * [lightBackground] and [lightToggle] are colors which control the
  ///   foreground and background colors representing the "light" theme mode
  /// * [darkBackground] and [darkToggle] are colors which control the
  ///   foreground and background colors representing the "dark" theme mode
  ///
  const Og1ThemeModeSelector({
    Key? key,
    this.durationInMs = 750,
    this.lightBackgroundColor,
    this.lightToggleColor,
    this.darkBackgroundColor,
    this.darkToggleColor,
    this.height = 39,
    required this.onChanged,
    required this.themeMode,
  }) : super(key: key);

  @override
  State<Og1ThemeModeSelector> createState() => _Og1ThemeModeSelectorState();
}

class _Og1ThemeModeSelectorState extends State<Og1ThemeModeSelector>
    with TickerProviderStateMixin {
  late ThemeModeSelectorConsts consts;
  late AnimationController _animationController;
  final Set<MaterialState> _states = {};

  late Animation<Alignment> _alignmentAnimation;
  late Animation<double> _starFade;
  late Animation<double> _flareFade;
  late Animation<double> _starToggleFade;
  late Animation<double> _flareToggleFade;
  late Animation<Color?> _bgColorAnimation;

  late bool isChecked;

  @override
  void initState() {
    super.initState();
    consts = ThemeModeSelectorConsts(widget.height);

    // Setup the global animation controller using the duration parameter
    _initializeAnimationController();
    isChecked = widget.themeMode == ThemeMode.dark;
    if (isChecked) {
      _animationController.forward();
    }
  }

  @override
  void didUpdateWidget(Og1ThemeModeSelector oldWidget) {
    super.didUpdateWidget(oldWidget);

    bool isCheckedUpdated = widget.themeMode == ThemeMode.dark;
    if (widget.themeMode != oldWidget.themeMode &&
        isChecked != isCheckedUpdated) {
      // rebuild the global animation controller using the duration parameter
      _initializeAnimationController();
      setState(() => isChecked = isCheckedUpdated);
      if (isCheckedUpdated) {
        _animationController.forward();
      } else {
        _animationController.reverse();
      }
    }
  }

  _initializeAnimationController() {
    Duration duration = Duration(milliseconds: widget.durationInMs);

    _animationController = AnimationController(
      vsync: this,
      duration: duration,
    );
  }

  initialize(BuildContext context, ThemeModeSelectorThemeData myTheme) {
    // Setup the tween for the background colors
    _bgColorAnimation = ColorTween(
      begin: lightBackgroundColor(myTheme),
      end: darkBackgroundColor(myTheme),
    ).animate(_animationController);

    // the tween for the toggle button (left and right)
    _alignmentAnimation = AlignmentTween(
      begin: Alignment.centerLeft,
      end: Alignment.centerRight,
    ).animate(
      CurvedAnimation(
        parent: _animationController,
        curve: const Interval(0.0, 0.9, curve: Curves.easeOutBack),
        reverseCurve: const Interval(0.0, 0.9, curve: Curves.easeInBack),
      ),
    );

    // Tweens and animation for the stars and flares
    var earlyFade = CurvedAnimation(
      parent: _animationController,
      curve: const Interval(0.0, 0.5, curve: Curves.elasticOut),
      reverseCurve: const Interval(0.5, 1.0, curve: Curves.elasticIn),
    );

    _starFade = Tween(begin: 0.0, end: 1.0).animate(earlyFade);
    _flareFade = Tween(begin: 1.0, end: 0.0).animate(earlyFade);
    _starToggleFade = Tween(begin: 0.0, end: 1.0).animate(earlyFade);
    _flareToggleFade = Tween(begin: 1.0, end: 0.0).animate(earlyFade);
  }

  @override
  void dispose() {
    _animationController.dispose();
    super.dispose();
  }

  // Builds the semi-complex tween for the stars and flares which aninate to
  // and fro from the center of the widget
  Animation<RelativeRect> slide(Offset from, Offset to, double size) {
    var container =
        Rect.fromLTWH(0, 0, consts.inset.width, consts.inset.height);
    return RelativeRectTween(
      begin: RelativeRect.fromRect(
          Rect.fromLTWH(from.dx, from.dy, size, size), container),
      end: RelativeRect.fromRect(
          Rect.fromLTWH(to.dx, to.dy, size, size), container),
    ).animate(
        // _animationController
        CurvedAnimation(
      parent: _animationController,
      curve: Curves.easeOutBack,
      reverseCurve: Curves.easeOut.flipped,
    ));
  }

  lightToggleColor(myTheme) =>
      widget.lightToggleColor ?? myTheme.lightToggleColor ?? Colors.white;

  lightBackgroundColor(myTheme) =>
      widget.lightBackgroundColor ??
      myTheme.lightBackgroundColor ??
      const Color(0xFF689DFF);

  darkToggleColor(myTheme) =>
      widget.darkToggleColor ?? myTheme.darkToggleColor ?? Colors.white;

  darkBackgroundColor(myTheme) =>
      widget.darkBackgroundColor ??
      myTheme.darkBackgroundColor ??
      const Color(0xFF040507);

  void _handleFocusHighlight(bool value) {
    bool modified = value
        ? _states.add(MaterialState.focused)
        : _states.remove(MaterialState.focused);
    if (modified) setState(() {});
  }

  void _handleHoverHighlight(bool value) {
    bool modified = value
        ? _states.add(MaterialState.hovered)
        : _states.remove(MaterialState.hovered);
    if (modified) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    ThemeModeSelectorThemeData myTheme = ThemeModeSelectorTheme.of(context);
    initialize(context, myTheme);

    return FocusableActionDetector(
      onShowFocusHighlight: _handleFocusHighlight,
      onShowHoverHighlight: _handleHoverHighlight,
      child: AnimatedBuilder(
        animation: _animationController,
        builder: (context, child) {
          return GestureDetector(
            onTap: () {
              if (_animationController.isCompleted) {
                _animationController.reverse();
                widget.onChanged(ThemeMode.light);
              } else {
                _animationController.forward();
                widget.onChanged(ThemeMode.dark);
              }
              setState(() {
                isChecked = !isChecked;
              });
            },
            child: Stack(
              alignment: Alignment.center,
              children: [
                Container(
                  width: consts.size.width,
                  height: consts.size.height,
                  padding: consts.padding,
                  decoration: BoxDecoration(
                    color: _bgColorAnimation.value,
                    borderRadius: BorderRadius.all(
                      Radius.circular(consts.size.height),
                    ),
                  ),
                  child: Stack(
                    children: <Widget>[
                      ...consts.stars
                          .map((star) => CelestialTransition(
                                alphaAnimation: _starFade,
                                relativeRectAnimation: slide(
                                    star['from'], star['to'], star['size']),
                                child: Star(
                                    size: star['size'],
                                    color: lightToggleColor(myTheme)),
                              ))
                          .toList(),
                      ...consts.flares
                          .map((flare) => CelestialTransition(
                                alphaAnimation: _flareFade,
                                relativeRectAnimation: slide(
                                    flare['from'], flare['to'], flare['size']),
                                child: Flare(
                                    size: flare['size'],
                                    color: lightToggleColor(myTheme)),
                              ))
                          .toList(),
                      Align(
                        alignment: _alignmentAnimation.value,
                        child: Stack(children: [
                          FadeTransition(
                            opacity: _flareToggleFade,
                            child: Sun(
                              color: lightToggleColor(myTheme),
                              size: consts.inset.height,
                            ),
                          ),
                          FadeTransition(
                            opacity: _starToggleFade,
                            child: Moon(
                              color: darkToggleColor(myTheme),
                              size: consts.inset.height,
                            ),
                          ),
                        ]),
                      ),
                    ],
                  ),
                ),
                FocusBackground(
                  padding: consts.focusPadding,
                  focused: _states.contains(MaterialState.focused),
                  width: consts.size.width,
                  height: consts.size.height,
                ),
              ],
            ),
          );
        },
      ),
    );
  }
}
